GLLParseError

Enum GLLParseError 

pub enum GLLParseError<'a> {
    UnexpectedByte {
        pointer: usize,
        expected: u8,
        offender: u8,
    },
    TooLong {
        pointer: usize,
        offender: Terminal<'a>,
    },
    NoCandidates {
        pointer: usize,
        rule: String,
        context: String,
    },
    ZeroWeights {
        pointer: usize,
        rule: String,
        context: String,
    },
}
Errors possible while GLL parsing.

Variants



UnexpectedByte

Encountered an unexpected byte.

Fields

pointer: usize

Where in the string the byte was encountered.

expected: u8

What we expected to see.

offender: u8

What we got.



TooLong

Expected more bytes than found in the input string.

Fields

pointer: usize

Where we are at the input string.

offender: Terminal<'a>

What character we expected to see.



NoCandidates

All the alternatives to this rule failed the first_set check.

Fields

pointer: usize

Where in the input stream this happened.

rule: String

String representation of the rule it happened in.

context: String

String representation of the parse context at this time.



ZeroWeights

All the alternatives to this rule had a 0 weight.

Fields

pointer: usize

Where in the input stream this happened.

rule: String

String representation of the rule it happened in.

context: String

String representation of the parse context at this time.