pub enum GLLProcessError {
MissingSPPFNode(SPPFNodeIndex),
}Expand description
Errors that can occur when processing the finished state object.
Variants
MissingSPPFNode(SPPFNodeIndex)
An SPPF node that we expect to exist in the graph is inexplicably missing.