RegexTerminal

Struct RegexTerminal 

pub struct RegexTerminal<'a> {
    pub automaton: DFA<&'a [u32]>,
    /* private fields */
}
A special type of Terminal which is a regex recognizer.

Implements label so that regex machines can be used. The string representation/uuid of the machine is its regex pattern.

Fields

automaton: DFA<&'a [u32]>

The regex automaton this terminal represents.

Implementations

Source

impl<'a> RegexTerminal<'a>

pub const fn new(pattern: &'a str, automaton: DFA<&'a [u32]>) -> Self

Construct a new RegexTerminal.