pub type RegexMap<'a> = HashMap<&'a str, Rc<RegexTerminal<'a>>>;Expand description
A map from a regular expression to the RegexTerminal that it represents.
Aliased Type
pub struct RegexMap<'a> { /* private fields */ }pub type RegexMap<'a> = HashMap<&'a str, Rc<RegexTerminal<'a>>>;A map from a regular expression to the RegexTerminal that it represents.
pub struct RegexMap<'a> { /* private fields */ }