pub enum ValueError<'a> {
ValueError(ValueError<Value<'a>>),
ConvertToLabel(Value<'a>),
}Expand description
An extension of wagon_value::ValueError for specific errors related to dealing with GLLBlockLabel.
Variants
ValueError(ValueError<Value<'a>>)
Any regular wagon_value::ValueError
ConvertToLabel(Value<'a>)
An error occured trying to convert a Value to a GLLBlockLabel.