pub enum GLLParseError<'a> {
UnexpectedByte {
pointer: usize,
expected: u8,
offender: u8,
},
TooLong {
pointer: usize,
offender: Terminal<'a>,
},
NoCandidates {
pointer: usize,
rule: String,
context: String,
},
ZeroWeights {
pointer: usize,
rule: String,
context: String,
},
}Expand description
Errors possible while GLL parsing.
Variants
UnexpectedByte
Encountered an unexpected byte.
Fields
TooLong
Expected more bytes than found in the input string.
Fields
NoCandidates
All the alternatives to this rule failed the first_set check.
Fields
ZeroWeights
All the alternatives to this rule had a 0 weight.